use std::sync::Arc;
use vulkano::buffer::{Buffer, BufferCreateInfo, BufferUsage, Subbuffer};
use vulkano::command_buffer::{AutoCommandBufferBuilder, PrimaryAutoCommandBuffer};
use vulkano::device::Device;
use vulkano::memory::allocator::{AllocationCreateInfo, MemoryTypeFilter, StandardMemoryAllocator};
use vulkano::pipeline::GraphicsPipeline;
use vulkano::swapchain::Swapchain;

use crate::renderer::{Vertex2D, create_triangle_pipeline};

pub struct Scene {
    pipeline: Arc<GraphicsPipeline>,
    vertex_buffer: Subbuffer<[Vertex2D]>,
}

impl Scene {
    fn create_vertex_buffer(
        memory_allocator: &Arc<StandardMemoryAllocator>,
    ) -> Subbuffer<[Vertex2D]> {
        let vertices = [
            // Triangle en haut à gauche
            Vertex2D::new([-0.5, -0.75],  // Coin supérieur gauche
                         [1.0, 0.0, 0.0]), // Couleur rouge
            Vertex2D::new([-0.75, -0.25],  // Coin supérieur
                         [0.0, 1.0, 0.0]), // Couleur verte
            Vertex2D::new([-0.25, -0.25],  // Coin supérieur droit
                         [0.0, 0.0, 1.0]), // Couleur bleue

            // Triangle en bas à gauche
            Vertex2D::new([-0.5, 0.25],    // Coin inférieur gauche
                         [0.5, 0.5, 0.5]), // Couleur gris
            Vertex2D::new([-0.75, 0.75],   // Coin inférieur
                         [0.2, 0.8, 0.2]), // Couleur vert clair
            Vertex2D::new([-0.25, 0.75],   // Coin inférieur droit
                         [0.8, 0.2, 0.2]), // Couleur rouge clair

            // Triangle en haut à droite
            Vertex2D::new([0.5, -0.75],    // Coin gauche supérieur
                         [1.0, 1.0, 0.0]), // Couleur jaune
            Vertex2D::new([0.25, -0.25],   // Coin gauche
                         [0.0, 1.0, 1.0]), // Couleur cyan
            Vertex2D::new([0.75, -0.25],   // Coin gauche inférieur
                         [1.0, 0.0, 1.0]), // Couleur magenta

            // Triangle en bas à droite
            Vertex2D::new([0.5, 0.25],     // Coin droit supérieur
                         [0.1, 0.5, 0.8]), // Couleur bleu clair
            Vertex2D::new([0.25, 0.75],    // Coin droit
                         [0.8, 0.6, 0.1]), // Couleur or
            Vertex2D::new([0.75, 0.75],    // Coin droit inférieur
                         [0.3, 0.4, 0.6]), // Couleur bleu foncé
        ];

        Buffer::from_iter(
            memory_allocator.clone(),
            BufferCreateInfo {
                usage: BufferUsage::VERTEX_BUFFER,
                ..Default::default()
            },
            AllocationCreateInfo {
                memory_type_filter: MemoryTypeFilter::PREFER_DEVICE
                    | MemoryTypeFilter::HOST_SEQUENTIAL_WRITE,
                ..Default::default()
            },
            vertices,
        )
        .unwrap()
    }

    pub fn initialize(
        device: &Arc<Device>,
        swapchain: &Arc<Swapchain>,
        memory_allocator: &Arc<StandardMemoryAllocator>,
    ) -> Scene {
        let pipeline = create_triangle_pipeline(device, swapchain);
        let vertex_buffer = Self::create_vertex_buffer(memory_allocator);

        Scene {
            pipeline,
            vertex_buffer,
        }
    }

    pub fn render(&self, builder: &mut AutoCommandBufferBuilder<PrimaryAutoCommandBuffer>) {
        builder
            .bind_pipeline_graphics(self.pipeline.clone())
            .unwrap()
            .bind_vertex_buffers(0, self.vertex_buffer.clone())
            .unwrap();

        let vertex_count = self.vertex_buffer.len() as u32;
        let instance_count = vertex_count / 3;

        unsafe { builder.draw(vertex_count, instance_count, 0, 0) }.unwrap();
    }
}